const express = require('express');
const path = require('path');
const logger = require('morgan');
const app = express();
const compression = require('compression');
const compressionCfg = {
    filter: () => {
        return true
    },
    threshold: 0
};

app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'pug');

app.use(logger('dev'));
app.use(require('./lib/headers').middleware);
app.use(require('./lib/deny-non-get'));
app.use(compression(compressionCfg));

app.use('/', express.static(path.join(__dirname, 'public')));

app.use('/', require('./routes/index'));
app.use('/', require('./routes/rhc'));

// catch 404 and forward to error handler
app.use((req, res, next) => {
    const err = new Error('Not Found');
    err.status = 404;
    next(err);
});

app.use((err, req, res, next) => {
    res.locals.message = err.message;
    res.locals.error = req.app.get('env') === 'development' ? err : {};
    res.status(err.status || 500);
    res.render('error');
});

module.exports = app;